# Copyright (c) 2014 Arista Networks, Inc.  All rights reserved.
# Arista Networks, Inc. Confidential and Proprietary.
import Url
from UrlPlugin.FlashUrl import FlashUrl
from UrlPlugin.FlashUrl import FlashFilesystem
import Tac
import errno
import os

sslConstants_ = Tac.Type( "Mgmt::Security::Ssl::Constants" )

def _isStandby( entityManager ):
   redundancyStatus = getattr( entityManager, 'redundancyStatus', None )
   if redundancyStatus and redundancyStatus().mode != 'active':
      return True
   return False

class SslUrl( FlashUrl ):
   def _checkStandby( self ):
      if ( self.fs.entityManager and _isStandby( self.fs.entityManager ) ):
         raise EnvironmentError( errno.EOPNOTSUPP, 
                                 "Operation not supported on this supervisor" )

   def put( self, srcFn, append=False ):
      self._checkStandby()
      FlashUrl.put( self, srcFn, append )

   def renameto( self, dst ):
      self._checkStandby()
      FlashUrl.renameto( self, dst )
      
   def delete( self, recursive ):
      self._checkStandby()
      FlashUrl.delete( self, recursive )
      
   def empty( self ):
      self._checkStandby()
      FlashUrl.empty( self )

   def writeLocalFile(self, srcUrl, filename):
      self._checkStandby()
      FlashUrl.writeLocalFile(self, srcUrl, filename)
   
   def listdir( self ):
      return os.listdir( self.realFilename_ )
      
class SslFileSystem( FlashFilesystem ):
   urlClass_ = SslUrl
   
   def __init__( self, scheme, rawlocation, permission, mask ):
      FlashFilesystem.__init__( self, scheme, rawlocation, fsType='ssl', 
                                permission=permission, mask=mask )

   def realFileSystem( self ):
      return True
   
   def stat( self ):
      return ( 0, 0 )
   
   def supportsMkdir( self ):
      return False

   def supportsListing( self ):
      return True
   
class SslCertFilesystem( SslFileSystem ):
   def __init__( self, scheme, rawlocation ):
      SslFileSystem.__init__( self, scheme, rawlocation, 'rw', 0002 )

   def validateFile( self, filename, durl=None, context=None ):
      import SslCertKey
      try:
         SslCertKey.validateCertificateOrCrl( filename )
      except SslCertKey.SslCertKeyError as e:
         raise EnvironmentError( 0, e )

class SslKeyFilesystem( SslFileSystem ):
   def __init__( self, scheme, rawlocation ):
      SslFileSystem.__init__( self, scheme, rawlocation, 'w', 0006 )

   def validateFile( self, filename, durl=None, context=None ):
      import SslCertKey
      try:
         SslCertKey.validateRsaPrivateKey( filename )
      except SslCertKey.SslCertKeyError as e:
         raise EnvironmentError( 0, e )
         
def Plugin( context=None ):
   Url.registerFilesystem( SslCertFilesystem( 'certificate:', 
                                              sslConstants_.certsDirPath() ) )
   Url.registerFilesystem( SslKeyFilesystem( 'sslkey:', 
                                              sslConstants_.keysDirPath() ) )
